/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

Ext.namespace('DSS.json');

/**
 * @class DSS.json.Base
 * @extends Ext.util.Observable
 * Abstract class that manages the json connection to the dSS.
 */
Ext.define('DSS.json.Base', {

    mixins: {
        observable: 'Ext.util.Observable'
    },

    uses: ['Ext.Ajax', 'Ext.JSON'],

    statics: {
        serialize: function(obj) {
            var s = '';
            var first = true;
            for (i in obj) {
                if (obj[i].length != 0) {
                    if (first) first = false;
                    else s += ';';
                    s += i.toString() + '=' + obj[i].toString().replace(';', '\\;');
                }
            }
            return s;
        }
    },

    constructor: function(config) {
        this.addEvents({
            ajaxFailure: true,
        });

        // Call the mixin's constructors
        this.mixins.observable.constructor.call(this, config);

        this.callParent(arguments);
    },

    request: function(params) {
        Ext.applyIf(params, {
            // default values to be used if not manually specified
            disableCaching: true,
            method: 'GET',
            scope: this,
            timeout: 30000, // 30s timeout
            failure: function(response, options) {
                this.fireEvent('ajaxFailure', response.responseText);
            }
        })
        Ext.Ajax.request(params);
    }
});

/**
 * @class DSS.json.Login
 * @extends Ext.util.Observable
 * Class that manages the json connection to the dSS to login.
 */
Ext.define('DSS.json.Login', {

    extend: 'DSS.json.Base',

    uses: ['Ext.JSON'],

    config: {
        user: 'dssadmin',
        password: 'dssadmin',
        loginUrl: '/json/system/login',
        loggedIn: false
    },

    constructor: function(config) {
        this.initConfig(config);

        this.addEvents({
            onLogin: true,
            loginSuccess: true,
            loginFailed: true
        });

        this.on({
            loginSuccess: function() { this.loggedIn = true; },
        });

        this.callParent(arguments);
    },

    login: function() {
        this.fireEvent('onLogin');
        this.request({
            url: this.loginUrl,
            params: {
                user: this.user,
                password: this.password
            },
            // scope: this,
            // method: 'GET',
            // disableCaching: true,
            success: function(response, options) {
                try {
                    if (response.responseText.length == 0) {
                        this.fireEvent('loginFailed', _('No response from dSS. Check that the dSS is running and reload this page again.'));
                        return;
                    }

                    var jsonData = Ext.JSON.decode(response.responseText);
                    if (jsonData.ok)
                        this.fireEvent('loginSuccess', jsonData.result);
                    else
                        this.fireEvent('loginFailed', jsonData.message);
                } catch (err) {
                    this.fireEvent('loginFailed', err.message);
                }
            }
        });
    }
});

/**
 * @class DSS.json.Property
 * @extends DSS.json.Base
 * Class that manages the json connection to manipulate dSS properties.
 * Note that it depends on DSS.json.Login in so far as the dSS requires the
 * connection to be logged in in order to execute queries
 */
Ext.define('DSS.json.Property', {

    extend: 'DSS.json.Base',

    uses: ['Ext.JSON'],

    config: {
        propertyUrl: '/json/property/',
        /*
        // Methods that take a path (not query!)
        pathMethods: [  'getBoolean', 'setBoolean',
                        'getInteger', 'setInteger',
                        'getString', 'setString',
                        'getChildren',
                        'getType',
                        'remove',
                        'setFlag'],
        */
        queryUrl: '/json/property/query'
    },

    constructor: function(config) {
        this.initConfig(config);
        this.addEvents({
            queryError: true,
            responseError: true
        });
        this.callParent(arguments);
    },

    ///
    /// Getters
    ///

    /**
     * Internal, do not call from outside
     */
    _getValue: function(req, resultHandlers) {
        if (Ext.isFunction(resultHandlers)) {
            resultHandlers = { success: resultHandlers };
        }
        if (resultHandlers.success !== undefined)
            req.success = function(resp) { this.defaultGetValueHandler(resultHandlers.success, resp); };
        if (resultHandlers.failure !== undefined)
            req.failure = resultHandlers.failure;
        this.request(req);
    },

    query: function(qry, resultHandlers) {
        var req = {
            url: this.queryUrl,
            params: { query: qry },
        };
        this._getValue(req, resultHandlers);
    },

    getBoolean: function(path, resultHandlers) {
        var req = {
            url: this.propertyUrl + 'getBoolean',
            params: { path: path },
        };
        this._getValue(req, resultHandlers);
    },

    getInteger: function(path, resultHandlers) {
        var req = {
            url: this.propertyUrl + 'getInteger',
            params: { path: path },
        };
        this._getValue(req, resultHandlers);
    },

    getString: function(path, resultHandlers) {
        var req = {
            url: this.propertyUrl + 'getString',
            params: { path: path },
        };
        this._getValue(req, resultHandlers);
    },

    /**
     * Gets the immediate children and their types asynchronously
     * @param path {String} The property path to fetch
     * @param resultHandlers {Object/Function} \
     *      Object with success and failure handlers. \
     *      Or simply the success handler
     */
    getChildren: function(path, resultHandlers) {
        var req = {
            url: this.propertyUrl + 'getChildren',
            params: { path: path },
        };
        this._getValue(req, resultHandlers);
    },

    getType: function(path, resultHandlers) {
        var req = {
            url: this.propertyUrl + 'getType',
            params: { path: path },
        };
        this._getValue(req, resultHandlers);
    },

    getFlags: function(path, resultHandlers) {
        var req = {
            url: this.propertyUrl + 'getFlags',
            params: { path: path },
        };
        this._getValue(req, resultHandlers);
    },

    ///
    /// Setters
    ///

    /**
     * Internal, do not call from outside
     */
    _setValue: function(req, callbacks) {
        if (callbacks.success !== undefined)
            req.success = function(resp) { this.defaultSetValueHandler(callbacks.success, resp); }
        if (callbacks.failure !== undefined)
            req.failure = resultHandlers.failure;
        this.request(req);
    },

    setBoolean: function(path, value, callbacks) {
        var req = {
            url: this.propertyUrl + 'setBoolean',
            params: { path: path, value: value },
        };
        this._setValue(req, callbacks);
    },

    setInteger: function(path, value, callbacks) {
        var req = {
            url: this.propertyUrl + 'setInteger',
            params: { path: path, value: value },
        };
        this._setValue(req, callbacks);
    },

    setString: function(path, value, callbacks) {
        var req = {
            url: this.propertyUrl + 'setString',
            params: { path: path, value: value },
        };
        this._setValue(req, callbacks);
    },

    setFlag: function(path, flag, callbacks) {
        var req = {
            url: this.propertyUrl + 'setFlag',
            params: { path: path, flag: flag },
        };
        this._setValue(req, callbacks);
    },
    setReadable: function(path, successCallback) {
        setFlag(path, 'READABLE', successCallback);
    },
    setWriteable: function(path, successCallback) {
        setFlag(path, 'WRITEABLE', successCallback);
    },
    setArchived: function(path, successCallback) {
        setFlag(path, 'ARCHIVE', successCallback);
    },

    ///
    /// Helper handlers
    ///

    /**
     * A helper handler that handles set* successes and
     * raises an alert box on error
     * @param callback {Function} The function to call on success
     * @param response {Object} The Xhr response object to check for success
     */
    defaultSetValueHandler: function(callback, response) {
        try {
            var data = Ext.JSON.decode(response.responseText);
            if (data.ok !== true) {
                this.fireEvent('queryError', data.message);
            }
        } catch(syntaxError) {
            this.fireEvent('responseError', syntaxError.message);
        }
    },

    /**
     * A helper handler that handles simple get* successes
     * (which return a single value) and raises an alert box
     * on error
     * Use like this:
     * success: function(response) { this.defaultGetValueHandler(this.myHandler, response); }
     * this.myHandler will then be called directly with the resulting value
     * @param valueHandler {Function} The handler to pass the parsed value to
     * @param response {Object} The Xhr response object to check for success
     */
    defaultGetValueHandler: function(valueHandler, response) {
        try {
            var data = Ext.JSON.decode(response.responseText);
            if (data.ok === true) {
                if (data.result !== undefined) {
                    if (data.result.value !== undefined)
                        valueHandler(data.result.value);
                    else
                        valueHandler(data.result);
                } else {
                    valueHandler(data.message);
                }
            } else {
                this.fireEvent('queryError', data.message);
            }
        } catch(syntaxError) {
            this.fireEvent('responseError', syntaxError.message);
        }
    }
});

/**
 * @class DSS.json.Event
 * @extends DSS.json.Base
 * Class that manages the json connection to manipulate dSS events.
 * Note that it depends on DSS.json.Login in so far as the dSS requires the
 * connection to be logged in in order to raise events
 */
Ext.define('DSS.json.Event', {

    extend: 'DSS.json.Base',

    uses: ['Ext.JSON'],

    config: {
        eventUrl: '/json/event/',
        eventName: '',
    },

    constructor: function(config) {
        this.initConfig(config);
        this.addEvents({
            eventError: true,
        });
        this.callParent(arguments);
    },

    raise: function(parameter, resultHandlers) {
        var req = {
            url: this.eventUrl + '/raise',
            params: {
                name: this.eventName,
                parameter: DSS.json.Base.serialize(parameter)
            }
        };
        if (resultHandlers !== undefined) {
            if (resultHandlers.success !== undefined)
                req.success = resultHandlers.success;
            if (resultHandlers.failure !== undefined)
                req.failure = resultHandlers.failure;
        }
        this.request(req);
    }
});

